# coding: utf-8

"""
    Togai Apis

    APIs for Togai App

    The version of the OpenAPI document: 1.0
    Contact: engg@togai.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from togai_client.models.accounts_billing_information import AccountsBillingInformation
from togai_client.models.address import Address
from togai_client.models.create_account_alias_request import CreateAccountAliasRequest
from togai_client.models.create_entity_setting import CreateEntitySetting
from typing import Optional, Set
from typing_extensions import Self

class CreateAccountRequestWithoutCustomerId(BaseModel):
    """
    Payload to create account
    """ # noqa: E501
    id: Annotated[str, Field(strict=True, max_length=50)] = Field(description="Identifier of the account")
    name: Annotated[str, Field(min_length=3, strict=True, max_length=255)] = Field(description="Name of the Account")
    invoice_currency: Optional[Annotated[str, Field(min_length=3, strict=True, max_length=3)]] = Field(default=None, description="Use [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217) currency code in which the account must be invoiced.   For example: AED is the currency code for United Arab Emirates dirham. ", alias="invoiceCurrency")
    aliases: Optional[Annotated[List[Annotated[str, Field(min_length=3, strict=True, max_length=50)]], Field(min_length=0, max_length=10)]] = Field(default=None, description="Aliases are tags that are associated with an account. Multiple aliases are allowed for a single account.")
    account_aliases: Optional[List[CreateAccountAliasRequest]] = Field(default=None, description="Aliases which effective range", alias="accountAliases")
    address: Optional[Address] = None
    primary_email: Optional[Annotated[str, Field(strict=True, max_length=320)]] = Field(default=None, description="Primary email of the account", alias="primaryEmail")
    billing_information: Optional[AccountsBillingInformation] = Field(default=None, alias="billingInformation")
    settings: Optional[Annotated[List[CreateEntitySetting], Field(min_length=0, max_length=10)]] = None
    net_term_days: Optional[StrictInt] = Field(default=None, description="Net term for the invoices of the account", alias="netTermDays")
    metadata: Optional[Dict[str, StrictStr]] = Field(default=None, description="Additional information associated with the account. Example: GSTN, VATN ")
    tags: Optional[List[StrictStr]] = Field(default=None, description="Tag for accounts are stored in lowercase")
    status: Optional[StrictStr] = Field(default=None, description="Status of the created account defaults to ACTIVE")
    __properties: ClassVar[List[str]] = ["id", "name", "invoiceCurrency", "aliases", "accountAliases", "address", "primaryEmail", "billingInformation", "settings", "netTermDays", "metadata", "tags", "status"]

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['ACTIVE', 'DRAFT']):
            raise ValueError("must be one of enum values ('ACTIVE', 'DRAFT')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateAccountRequestWithoutCustomerId from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in account_aliases (list)
        _items = []
        if self.account_aliases:
            for _item_account_aliases in self.account_aliases:
                if _item_account_aliases:
                    _items.append(_item_account_aliases.to_dict())
            _dict['accountAliases'] = _items
        # override the default output from pydantic by calling `to_dict()` of address
        if self.address:
            _dict['address'] = self.address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of billing_information
        if self.billing_information:
            _dict['billingInformation'] = self.billing_information.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in settings (list)
        _items = []
        if self.settings:
            for _item_settings in self.settings:
                if _item_settings:
                    _items.append(_item_settings.to_dict())
            _dict['settings'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateAccountRequestWithoutCustomerId from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "name": obj.get("name"),
            "invoiceCurrency": obj.get("invoiceCurrency"),
            "aliases": obj.get("aliases"),
            "accountAliases": [CreateAccountAliasRequest.from_dict(_item) for _item in obj["accountAliases"]] if obj.get("accountAliases") is not None else None,
            "address": Address.from_dict(obj["address"]) if obj.get("address") is not None else None,
            "primaryEmail": obj.get("primaryEmail"),
            "billingInformation": AccountsBillingInformation.from_dict(obj["billingInformation"]) if obj.get("billingInformation") is not None else None,
            "settings": [CreateEntitySetting.from_dict(_item) for _item in obj["settings"]] if obj.get("settings") is not None else None,
            "netTermDays": obj.get("netTermDays"),
            "metadata": obj.get("metadata"),
            "tags": obj.get("tags"),
            "status": obj.get("status")
        })
        return _obj


