# coding: utf-8

"""
    Togai Apis

    APIs for Togai App

    The version of the OpenAPI document: 1.0
    Contact: engg@togai.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from togai_client.models.computation import Computation
from togai_client.models.event_schema import EventSchema
from togai_client.models.usage_meter_aggregation import UsageMeterAggregation
from togai_client.models.usage_meter_filter_entry import UsageMeterFilterEntry
from typing import Optional, Set
from typing_extensions import Self

class UsageMeter(BaseModel):
    """
    Structure of usage meter
    """ # noqa: E501
    id: Annotated[str, Field(strict=True, max_length=20)] = Field(description="Identifier of the usage meter")
    name: Annotated[str, Field(strict=True, max_length=50)] = Field(description="Name of the usage meter")
    billable_name: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="Billable name of addon. Billable name takes precedence over name to display in invoice.", alias="billableName")
    display_name: Annotated[str, Field(strict=True, max_length=255)] = Field(description="Display name of usage meter. This is an auto-generated field which contains billableName of usage meter. If billableName is not provided, name will be used as display name. ", alias="displayName")
    description: Optional[Annotated[str, Field(strict=True, max_length=255)]] = None
    filters: Optional[List[UsageMeterFilterEntry]] = None
    type: StrictStr = Field(description="Type of usage meter")
    status: Optional[StrictStr] = Field(default=None, description="Status of usage meter")
    aggregation: UsageMeterAggregation
    computations: Optional[Annotated[List[Computation], Field(min_length=0, max_length=1)]] = None
    event_schema: Optional[EventSchema] = Field(default=None, alias="eventSchema")
    created_at: Optional[datetime] = Field(default=None, alias="createdAt")
    last_activated_at: Optional[datetime] = Field(default=None, alias="lastActivatedAt")
    updated_at: Optional[datetime] = Field(default=None, alias="updatedAt")
    __properties: ClassVar[List[str]] = ["id", "name", "billableName", "displayName", "description", "filters", "type", "status", "aggregation", "computations", "eventSchema", "createdAt", "lastActivatedAt", "updatedAt"]

    @field_validator('name')
    def name_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[\sa-zA-Z0-9_-]*$", value):
            raise ValueError(r"must validate the regular expression /^[\sa-zA-Z0-9_-]*$/")
        return value

    @field_validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['COUNTER']):
            raise ValueError("must be one of enum values ('COUNTER')")
        return value

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['DRAFT', 'ACTIVE', 'INACTIVE', 'ARCHIVED']):
            raise ValueError("must be one of enum values ('DRAFT', 'ACTIVE', 'INACTIVE', 'ARCHIVED')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UsageMeter from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in filters (list)
        _items = []
        if self.filters:
            for _item_filters in self.filters:
                if _item_filters:
                    _items.append(_item_filters.to_dict())
            _dict['filters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in computations (list)
        _items = []
        if self.computations:
            for _item_computations in self.computations:
                if _item_computations:
                    _items.append(_item_computations.to_dict())
            _dict['computations'] = _items
        # override the default output from pydantic by calling `to_dict()` of event_schema
        if self.event_schema:
            _dict['eventSchema'] = self.event_schema.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UsageMeter from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "name": obj.get("name"),
            "billableName": obj.get("billableName"),
            "displayName": obj.get("displayName"),
            "description": obj.get("description"),
            "filters": [UsageMeterFilterEntry.from_dict(_item) for _item in obj["filters"]] if obj.get("filters") is not None else None,
            "type": obj.get("type"),
            "status": obj.get("status"),
            "aggregation": obj.get("aggregation"),
            "computations": [Computation.from_dict(_item) for _item in obj["computations"]] if obj.get("computations") is not None else None,
            "eventSchema": EventSchema.from_dict(obj["eventSchema"]) if obj.get("eventSchema") is not None else None,
            "createdAt": obj.get("createdAt"),
            "lastActivatedAt": obj.get("lastActivatedAt"),
            "updatedAt": obj.get("updatedAt")
        })
        return _obj


